package edu.kit.ipd.descartes.adaptation.evaluation.performance;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;

import edu.kit.ipd.descartes.adaptation.evaluation.performance.weightingfunction.IWeightingFunction;
import edu.kit.ipd.descartes.adaptation.evaluation.performance.weightingfunction.WeightedSumWeightingFunctionHelper;
import edu.kit.ipd.descartes.mm.adaptation.WeightedTactic;
import edu.kit.ipd.descartes.mm.adaptation.WeightingFunction;
import edu.kit.ipd.descartes.perfdatarepo.Impact;
import edu.kit.ipd.descartes.perfdatarepo.MetricValue;
import edu.kit.ipd.descartes.perfdatarepo.Result;

/**
 * Evaluates a given {@link WeightedTactic} using the weighted tactic's {@link WeightingFunction}.
 * 
 * @author nhuber
 * 
 */
public class WeightingFunctionEvaluator implements IEvaluator {

    private static final String RESPONSETIME_IDENTIFIER = "ResponseTime";
    private IWeightingFunction calculator = null;
    private static Logger logger = Logger.getLogger(WeightingFunctionEvaluator.class);

    /*
     * (non-Javadoc)
     * 
     * @see edu.kit.ipd.descartes.adaptation.evaluation.IEvaluator#evaluate()
     */
    @Override
    public void evaluate(WeightedTactic tactic) {
        WeightingFunction f = tactic.getParentStrategy().getWeightingFunction();

        final String functionClassName = f.getWeightingFunctionImplementation();

        Object o = null;
        try {
            o = Class.forName(functionClassName).newInstance();
        } catch (final ClassNotFoundException e) {
            e.printStackTrace();
        } catch (final InstantiationException e) {
            e.printStackTrace();
        } catch (final IllegalAccessException e) {
            e.printStackTrace();
        }

        if (!IWeightingFunction.class.isInstance(o)) {
            throw new RuntimeException("Expected a class implementing " + IWeightingFunction.class.getName());
        }

        calculator = (IWeightingFunction) o;
        calculator.setWeightingFunction(f);
        double newWeight = calculator.calculateWeight(tactic);
        logger.debug("Setting new weight of " + tactic.getName() + " to " + newWeight);
        tactic.setCurrentWeight(newWeight);
    }

    public boolean slaViolated(WeightedTactic tactic) {
        Impact impact = tactic.getLastImpact();
        Result before = impact.getBefore();
        Result after = impact.getAfter();
        EList<MetricValue> afterMetricValues = after.getMetricValues();

        for (MetricValue afterMetricValue : afterMetricValues) {
            // calculated delta=(after-before)
            double afterValue = afterMetricValue.getValue();
            double beforeValue = WeightedSumWeightingFunctionHelper.getValueForMetricType(
                    afterMetricValue.getMetricType(), before);

            if (afterMetricValue.getMetricType().getName().contains(RESPONSETIME_IDENTIFIER)
                    && afterValue > beforeValue) {
                return true;
            }
        }

        return false;
    }

    @Override
    public void punishTactic(WeightedTactic tactic) {
        double weight = 0.0;
        logger.info("Punishing tactic, setting weight to " + weight);
        tactic.setCurrentWeight(weight);
    }
}
